/*******************************************************************************
* Copyright (c) 2017, 2018, 2019 in-tech GmbH
*               2018 AMFD GmbH
*               2016, 2017, 2018, 2020 ITK Engineering GmbH
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
*******************************************************************************/

//-----------------------------------------------------------------------------
//! @file  AgentExternalInterface.h
//! @brief 
//-----------------------------------------------------------------------------

#pragma once

#include <map>
#include <list>
#include <vector>

#include "Common/globalDefinitions.h"
#include "Common/worldDefinitions.h"
#include "Interfaces/worldObjectInterface.h"
#include "Interfaces/spawnPointInterface.h"
#include "Interfaces/signalInterface.h"
#include "Interfaces/profilesInterface.h"

struct DynamicState 
{
    double xPos;
    double yPos;
    double yaw;
    
    double xVel;
    double yVel;
    double yawVel;
    
    double xAcc;
    double yAcc;
    double yawAcc;
};

enum class ReferenceFrame
{
    World = 0;
    AgentStaticFrame; // origin in the vehicle position (AgentReferencePoint), static relative to the world frame
};

enum class ReferencePoint
{
    GeometricalCenter = 0; // Center of length and width
    CenterOfMass;
    CenterOfRearAxle; // Only for vehicles
};

struct AgentSignals
{
    IndicatorState indicatorstate;
    LightState lightstate;
    bool brakeLight;
    bool horn;
    bool headlight;
    bool highBeamLight;
    bool flasher;
};

class AgentExternalInterface
{
public:
    AgentExternalInterface() = default;
    AgentExternalInterface(const AgentExternalInterface &) = delete;
    AgentExternalInterface(AgentExternalInterface &&) = delete;
    AgentExternalInterface &operator=(const AgentExternalInterface &) = delete;
    AgentExternalInterface &operator=(AgentExternalInterface &&) = delete;
    virtual ~AgentExternalInterface() = default;

    //-----------------------------------------------------------------------------
    //! Retrieves dynamic state of agent
    //!
    // @return                Dynamic State
    //-----------------------------------------------------------------------------
    virtual DynamicState GetDynamicState(ReferenceFrame frame, ReferencePoint refPoint) const = 0;

    //-----------------------------------------------------------------------------
    //! Retrieves state of external signals of the agent
    //!
    // @return                AgentSignals
    //-----------------------------------------------------------------------------
    virtual AgentSignals GetAgentSignals() const = 0;
        
    //-----------------------------------------------------------------------------
    //! Returns all the TrafficSigns valid for the given lane from the agent postion within the given search distance
    //!
    //! @return vector of traffic signs, or emtpy list if no signs are available
    //-----------------------------------------------------------------------------
    virtual std::vector<CommonTrafficSign::Entity> GetTrafficSignsInRange(double searchDistance, int relativeLane = 0) const = 0;

    //-----------------------------------------------------------------------------
    //! Returns all the LaneMarkings on the given side within the given search distance from the agent postion
    //!
    //! \param searchDistance   range of the search
    //! \param relativeLane     lane to look in relative to agent main lane
    //! \param side             side of the lane
    //-----------------------------------------------------------------------------
    virtual std::vector<LaneMarking::Entity> GetLaneMarkingsInRange(double searchDistance, int relativeLane, Side side) const = 0;

    //! Returns the relative distances (start and end) and the connecting road id of all junctions on the route in range
    //!
    //! \param range            range of search
    //! \return information about all junctions in range
    virtual RelativeWorldView::Junctions GetRelativeJunctions (double range) const = 0;

    //! Returns information about all lanes on the route in range. These info are the relative distances (start and end),
    //! the laneId relative to the ego lane, the successors and predecessors if existing and the information wether the intended
    //! driving direction of the lane is the same as the direction of the route. If the ego lane prematurely ends, then
    //! the further lane ids are relative to the middle of the road.
    //!
    //! \param range            range of search
    //! \return information about all lanes in range
    virtual RelativeWorldView::Lanes GetRelativeLanes (double range) const = 0;

}