/*******************************************************************************
* Copyright (c) 2017, 2018, 2019 in-tech GmbH
*               2018 AMFD GmbH
*               2016, 2017, 2018, 2020 ITK Engineering GmbH
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
*******************************************************************************/

//-----------------------------------------------------------------------------
//! @file  AgentFrameworkInterface.h
//! @brief 
//-----------------------------------------------------------------------------

#pragma once


class AgentFrameworkInterface
{
public:
    AgentFrameworkInterface() = default;
    AgentFrameworkInterface(const AgentFrameworkInterface &) = delete;
    AgentFrameworkInterface(AgentFrameworkInterface &&) = delete;
    AgentFrameworkInterface &operator=(const AgentFrameworkInterface &) = delete;
    AgentFrameworkInterface &operator=(AgentFrameworkInterface &&) = delete;
    virtual ~AgentFrameworkInterface() = default;

    //-----------------------------------------------------------------------------
    //! Inits all physical and world specific parameters of an agent.
    //!
    // @return     true for success
    //-----------------------------------------------------------------------------
    virtual bool InitAgentParameter(int id,
                                    int agentTypeId,
                                    int spawnTime,
                                    const AgentSpawnItem *agentSpawnItem,
                                    const SpawnItemParameterInterface &spawnItemParameter) = 0;

    //-----------------------------------------------------------------------------
    //! Inits all physical and world specific parameters of an agent.
    //!
    //! @return    true for success
    //-----------------------------------------------------------------------------
    virtual bool InitAgentParameter(int id,
                                    int spawnTime,
                                    AgentBlueprintInterface* agentBlueprint) = 0;

    //-----------------------------------------------------------------------------
    //! Checks if an agent is still valid or if its marked for remove.
    //!
    // @return     true if still valid, false if marked for remove
    //-----------------------------------------------------------------------------
    virtual bool IsValid() const = 0;

    //-----------------------------------------------------------------------------
    //! Unregisters the agent from the world.
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual void Unregister() const = 0;

    //-----------------------------------------------------------------------------
    //! Unlocate agent in world.
    //!
    // @return
    //-----------------------------------------------------------------------------
    virtual void Unlocate() = 0;

    //-----------------------------------------------------------------------------
    //! Locate agent in world.
    //!
    // @return true, if agent could be localized
    //-----------------------------------------------------------------------------
    virtual bool Locate() = 0;

    //-----------------------------------------------------------------------------
    //! Update agent for in new timestep
    //!
    //! @return true, if agent could be localized
    //-----------------------------------------------------------------------------
    virtual bool Update() = 0;

    //-----------------------------------------------------------------------------
    //! Returns true if agent is still in World located.
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual bool IsAgentInWorld() const = 0;


    //-----------------------------------------------------------------------------
    //! Requests removal of agents at next time step.
    //-----------------------------------------------------------------------------
    virtual void RemoveAgent() = 0;
        
    //-----------------------------------------------------------------------------
    //! Sets all vehicle model parameters of agent
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual void SetVehicleModelParameters(VehicleModelParameters vehicleModelParameters) = 0;
    
    //-----------------------------------------------------------------------------
    //! Retrieves list of collisions partners of agent.
    //!
    //! @return               a value >0 if collided
    //-----------------------------------------------------------------------------
    virtual std::vector<std::pair<ObjectTypeOSI, int>> GetCollisionPartners() const = 0;

    //-----------------------------------------------------------------------------
    //! update list with collision partners
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual void UpdateCollision(std::pair<ObjectTypeOSI, int> collisionPartner) = 0;
    
};
