/*******************************************************************************
* Copyright (c) 2017, 2018, 2019 in-tech GmbH
*               2018 AMFD GmbH
*               2016, 2017, 2018, 2020 ITK Engineering GmbH
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
*******************************************************************************/

//-----------------------------------------------------------------------------
//! @file  AgentInternalInterface.h
//! @brief 
//-----------------------------------------------------------------------------

#pragma once

struct InternalStaticState
{
    double spawnTime; // or obsolete
    std::string DriverProfileName;
    std::string AgentTypeName;
    std::string ScenarioName;
    AgentCategory agentCategory;
};

struct InternalDynamicState
{
    int gear;
    double steeringWheelAngle;
    double distanceTraveled;
    double engineSpeed;
    double speedGoalMin;
    double effAccelPedal;
    double effBrakePedal;
};

class AgentInternalInterface : public virtual agentExternalInterface
{
public:
    AgentInternalInterface() = default;
    AgentInternalInterface(const AgentInternalInterface &) = delete;
    AgentInternalInterface(AgentInternalInterface &&) = delete;
    AgentInternalInterface &operator=(const AgentInternalInterface &) = delete;
    AgentInternalInterface &operator=(AgentInternalInterface &&) = delete;
    virtual ~AgentInternalInterface() = default;
      
    //-----------------------------------------------------------------------------
    //! Retrieves id of agent
    //!
    // @return                Id of agent
    //-----------------------------------------------------------------------------
    virtual int GetAgentId() const = 0;

    //-----------------------------------------------------------------------------
    //! Returns true if Agent is marked as the ego agent.
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual bool IsEgoAgent() const = 0;
    
    //-----------------------------------------------------------------------------
    //! Retrieves the type key of an agent
    //!
    //! @return                string identifier of vehicle type
    //-----------------------------------------------------------------------------
    virtual std::string GetVehicleModelType() const = 0;

    //-----------------------------------------------------------------------------
    //! Retrieves all vehicle model parameters of agent
    //!
    //! @return               VehicleModelParameters of agent
    //-----------------------------------------------------------------------------
    virtual VehicleModelParameters GetVehicleModelParameters() const = 0;
    
    virtual const std::list<SensorParameter>& GetSensorParameters() const = 0;

    //-----------------------------------------------------------------------------
    //! Retrieves internal static state of vehicle
    //!
    //! @param[in]     InternalStaticState
    //-----------------------------------------------------------------------------
    virtual InternalStaticState GetInternalStaticState() const = 0;

    //-----------------------------------------------------------------------------
    //! Retrieves internal dynamic state of vehicle
    //!
    //! @param[in]     InternalDynamicState
    //-----------------------------------------------------------------------------
    virtual InternalDynamicState GetInternalDynamicState() const = 0;
    
    //-----------------------------------------------------------------------------
    //! Set dynamic state of agent
    //!
    //-----------------------------------------------------------------------------
    virtual void SetDynamicState(ReferenceFrame frame, ReferencePoint refPoint) = 0;

    //-----------------------------------------------------------------------------
    //! Sets state of external signals of the agent
    //!
    // @return                AgentSignals
    //-----------------------------------------------------------------------------
    virtual void SetAgentSignals(AgentSignals) = 0;

    virtual void SetSensorParameters(std::list<SensorParameter> sensorParameters) = 0;

    //-----------------------------------------------------------------------------
    //! Sets internal static state of vehicle
    //!
    //! @param[in]     InternalStaticState
    //-----------------------------------------------------------------------------
    virtual void SetInternalStaticState(InternalStaticState internalStaticState) = 0;

    //-----------------------------------------------------------------------------
    //! Sets internal dynamic state of vehicle
    //!
    //! @param[in]     InternalDynamicState
    //-----------------------------------------------------------------------------
    virtual void SetInternalDynamicState(InternalDynamicState internalDynamicState) = 0;

    virtual const Route& GetRoute() const = 0;
    
        //-----------------------------------------------------------------------------
    //! Returns the AgentExternalInterface of the next agent in front in a specific lane.
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual const AgentExternalInterface *GetAgentInFront(int laneId) const = 0;

    //-----------------------------------------------------------------------------
    //! Returns the AgentExternalInterface of the next agent behind in a specific lane.
    //!
    //! @return
    //-----------------------------------------------------------------------------
    virtual const AgentExternalInterface *GetAgentBehind(int laneId) const = 0;

    //! Returns a vector with all AgentExternalInterface of the next agents in front in all assigned lanes.
    virtual std::vector<AgentExternalInterface*> GetAllAgentsInFront() const = 0;

    //-----------------------------------------------------------------------------
    //! Returns all agents around the agent on given lane inside the given range
    //! Agents partially in range are also considered
    //!
    //! @param[in]      relativeLane    lane relative to agent main lane, e.g. -1 for right lane
    //! @param[in]      backwardRange   range in backwards direction (>= 0)
    //! @param[in]      forwardRange    range in forward direction (>= 0)
    //! @param[in]      mp              measurement point for range
    //-----------------------------------------------------------------------------
    virtual std::vector<const AgentExternalInterface *> GetAgentsInRange(int relativeLane, double backwardRange, double forwardRange, MeasurementPoint mp) const = 0;
}